keyboard {

    ubyte[128] state = $80
    uword old_keyhdl = 0

    sub init() {
        reset()
        sys.set_irqd()
        old_keyhdl = cx16.KEYHDL
        cx16.KEYHDL = &keyboard_handler
        sys.clear_irqd()
    }

    sub deinit() {
        if old_keyhdl == 0 return

        sys.set_irqd()
        cx16.KEYHDL = old_keyhdl
        sys.clear_irqd()
        reset()
    }

    sub reset() {
        ubyte i
        for i in 0 to 127 
        {
            state[i] = $80
        }
    }

    sub keyboard_handler(ubyte keynum) -> ubyte {

        state[keynum & $7f] = keynum & $80
        return keynum
    }

    sub pressed(ubyte key) -> bool {

        if (state[key] & $80) == 0 return true
        return false
    }

    sub pressed_release(ubyte key) -> bool {

        ubyte keystate = state[key] & $80
        state[key] = $80
        if keystate == 0 return true
        return false
    }
}


; KEYCODE_BACKTICK    = $01       ; 29                        0E                              07 35       `
; KEYCODE_1           = $02       ; 02                        16                              07 1E       1
; KEYCODE_2           = $03       ; 03                        1E                              07 1F       2
; KEYCODE_3           = $04       ; 04                        26                              07 20       3
; KEYCODE_4           = $05       ; 05                        25                              07 21       4
; KEYCODE_5           = $06       ; 06                        2E                              07 22       5
; KEYCODE_6           = $07       ; 07                        36                              07 23       6
; KEYCODE_7           = $08       ; 08                        3D                              07 24       7
; KEYCODE_8           = $09       ; 09                        3E                              07 25       8
; KEYCODE_9           = $0A       ; 0A                        46                              07 26       9
; KEYCODE_0           = $0B       ; 0B                        45                              07 27       0
; KEYCODE_MINUS       = $0C       ; 0C                        4E                              07 2D       -
; KEYCODE_EQUAL       = $0D       ; 0D                        55                              07 2E       =
; KEYCODE_BACKSPACE   = $0F       ; 0E                        66                              07 2A       Backspace
; KEYCODE_TAB         = $10       ; 0F                        0D                              07 2B       Tab
; KEYCODE_Q           = $11       ; 10                        15                              07 14       q
; KEYCODE_W           = $12       ; 11                        1D                              07 1A       w
; KEYCODE_E           = $13       ; 12                        24                              07 08       e
; KEYCODE_R           = $14       ; 13                        2D                              07 15       r
; KEYCODE_T           = $15       ; 14                        2C                              07 17       t
; KEYCODE_Y           = $16       ; 15                        35                              07 1C       y
; KEYCODE_U           = $17       ; 16                        3C                              07 18       u
; KEYCODE_I           = $18       ; 17                        43                              07 0C       i
; KEYCODE_O           = $19       ; 18                        44                              07 12       o
; KEYCODE_P           = $1A       ; 19                        4D                              07 13       p
; KEYCODE_LBRACKET    = $1B       ; 1A                        54                              07 2F       [
; KEYCODE_RBRACKET    = $1C       ; 1B                        5B                              07 30       ]
; KEYCODE_BACKSLASH   = $1D       ; 2B                        5D                              07 31       \
; KEYCODE_CAPSLOCK    = $1E       ; 3A                        58                              07 39       Caps Lock
; KEYCODE_A           = $1F       ; 1E                        1C                              07 04       a
; KEYCODE_S           = $20       ; 1F                        1B                              07 16       s
; KEYCODE_D           = $21       ; 20                        23                              07 07       d
; KEYCODE_F           = $22       ; 21                        2B                              07 09       f
; KEYCODE_G           = $23       ; 22                        34                              07 0A       g
; KEYCODE_H           = $24       ; 23                        33                              07 0B       h
; KEYCODE_J           = $25       ; 24                        3B                              07 0D       j
; KEYCODE_K           = $26       ; 25                        42                              07 0E       k
; KEYCODE_L           = $27       ; 26                        4B                              07 0F       l
; KEYCODE_SEMICOLON   = $28       ; 27                        4C                              07 33       ;
; KEYCODE_SINGLEQUOTE = $29       ; 28                        52                              07 34       '
; KEYCODE_ENTER       = $2B       ; 1C                        5A                              07 28       Enter
; KEYCODE_LSHIFT      = $2C       ; 2A                        12                              07 E1       Left Shift
; KEYCODE_NONUSLESS   = $2D       ; 56                        61                              07 64       Key not in US layout. On a 102-key keyboard, right of left Shift key, typically "<".
; KEYCODE_Z           = $2E       ; 2C                        1A                              07 1D       z
; KEYCODE_X           = $2F       ; 2D                        22                              07 1B       x
; KEYCODE_C           = $30       ; 2E                        21                              07 06       c
; KEYCODE_V           = $31       ; 2F                        2A                              07 19       v
; KEYCODE_B           = $32       ; 30                        32                              07 05       b
; KEYCODE_N           = $33       ; 31                        31                              07 11       n
; KEYCODE_M           = $34       ; 32                        3A                              07 10       m
; KEYCODE_COMMA       = $35       ; 33                        41                              07 36       ,
; KEYCODE_POINT       = $36       ; 34                        49                              07 37       .
; KEYCODE_SLASH       = $37       ; 35                        4A                              07 38       /
; KEYCODE_INTL1       = $38       ; 73                        51                              07 87       Keyboard Int'l 1
; KEYCODE_RSHIFT      = $39       ; 36                        59                              07 E5       Right Shift
; KEYCODE_LCTRL       = $3A       ; 1D                        14                              07 E0       Left Ctrl
; KEYCODE_LGUI        = $3B       ; E0 5B                     E0 1F                           07 E3       Left GUI
; KEYCODE_LALT        = $3C       ; 38                        11                              07 E2       Left Alt
; KEYCODE_SPACE       = $3D       ; 39                        29                              07 2C       Spacebar
; KEYCODE_RALT        = $3E       ; E0 38                     E0 11                           07 E6       Right Alt
; KEYCODE_RGUI        = $3F       ; E0 5C                     E0 27                           07 E7       Right GUI
; KEYCODE_RCTRL       = $40       ; E0 1D                     E0 14                           07 E4       Right Ctrl
; KEYCODE_MENU        = $41       ; ---			    E0 2F			    07 76	Menu
; KEYCODE_INSERT      = $4B       ; E0 52                     E0 70                           07 49       Insert
; KEYCODE_DELETE      = $4C       ; E0 4B                     E0 71                           07 4C       Delete
; KEYCODE_LEFTARROW   = $4F       ; E0 4B                     E0 6B                           07 50       Left Arrow
; KEYCODE_HOME        = $50       ; E0 47                     E0 6C                           07 4A       Home
; KEYCODE_END         = $51       ; E0 4F                     E0 69                           07 4D       End
; KEYCODE_UPARROW     = $53       ; E0 48                     E0 75                           07 52       Up Arrow
; KEYCODE_DOWNARROW   = $54       ; E0 50                     E0 72                           07 51       Down Arrow
; KEYCODE_PGUP        = $55       ; E0 49                     E0 7D                           07 4B       Page Up
; KEYCODE_PGDN        = $56       ; E0 51                     E0 7A                           07 4E       Page Down
; KEYCODE_RIGHTARROW  = $59       ; E0 4D                     E0 74                           07 4F       Right Arrow
; KEYCODE_NUMLOCK     = $5A       ; 45                        77                              07 53       Num Lock
; KEYCODE_KP7         = $5B       ; 47                        6C                              07 5F       Keypad 7
; KEYCODE_KP4         = $5C       ; 4B                        6B                              07 5C       Keypad 4
; KEYCODE_KP1         = $5D       ; 4F                        69                              07 59       Keypad 1
; KEYCODE_KPDIV       = $5F       ; E0 35                     E0 4A                           07 54       Keypad /
; KEYCODE_KP8         = $60       ; 48                        75                              07 60       Keypad 8
; KEYCODE_KP5         = $61       ; 4C                        73                              07 5D       Keypad 5
; KEYCODE_KP2         = $62       ; 50                        72                              07 5A       Keypad 2
; KEYCODE_KP0         = $63       ; 52                        70                              07 62       Keypad 0
; KEYCODE_KPMULT      = $64       ; 37                        7C                              07 55       Keypad *
; KEYCODE_KP9         = $65       ; 49                        7D                              07 61       Keypad 9
; KEYCODE_KP6         = $66       ; 4D                        74                              07 5E       Keypad 6
; KEYCODE_KP3         = $67       ; 51                        7A                              07 5B       Keypad 3
; KEYCODE_KPPOINT     = $68       ; 53                        71                              07 63       Keypad .
; KEYCODE_KPSUBTRACT  = $69       ; 4A                        7B                              07 56       Keypad -
; KEYCODE_KPADD       = $6A       ; 4E                        79                              07 57       Keypad +
; KEYCODE_KPENTER     = $6C       ; E0 1C                     E0 5A                           07 58       Keypad Enter
; KEYCODE_ESC         = $6E       ; 01                        76                              07 29       Esc
; KEYCODE_F1          = $70       ; 3B                        05                              07 3A       F1
; KEYCODE_F2          = $71       ; 3C                        06                              07 3B       F2
; KEYCODE_F3          = $72       ; 3D                        04                              07 3C       F3
; KEYCODE_F4          = $73       ; 3E                        0C                              07 3D       F4
; KEYCODE_F5          = $74       ; 3F                        03                              07 3E       F5
; KEYCODE_F6          = $75       ; 40                        0B                              07 3F       F6
; KEYCODE_F7          = $76       ; 41                        83                              07 40       F7
; KEYCODE_F8          = $77       ; 42                        0A                              07 41       F8
; KEYCODE_F9          = $78       ; 43                        01                              07 42       F9
; KEYCODE_F10         = $79       ; 44                        09                              07 43       F10
; KEYCODE_F11         = $7A       ; 57                        78                              07 44       F11
; KEYCODE_F12         = $7B       ; 58                        07                              07 45       F12
; KEYCODE_PRTSCR      = $7C       ; E0 2A E0 37               E0 12 E0 7C                     07 46       Print Screen
; KEYCODE_SCRLCK      = $7D       ; 46                        7E                              07 47       Scroll Lock
; KEYCODE_PAUSEBRK    = $7E       ; E1 1D 45 E1 9D C5         E1 14 77 E1 F0 14 F0 77         07 48       Pause Break